# final_review.py
# questions taken from various exams, cited as applicable throughout

def secret_message(msg):
    """
    Returns the secret messages inside of the msg.

    The secret message in msg will be the string after the first occurence of
    '<<' but before the last occurence of '>>' with white any potential leading
    and trailing spaces removed. There could be an unknown number of
    spaces between '<<', the message, and '>>'.

    For example,
        secret_message("urie>>anveovn<<      You're awesome!!!   >>vudbku<<")
    returns:
        'You're awesome!!!' (notice, leading and trailing spaces removed)

    And,
        secret_message("<<U R Cool >> >> >>")
    returns:
        'U R Cool >> >>' (notice, the message ends at the LAST occurence of '>>')

    Precondition: msg is a string with at least one occurence of '<<' before
        the last occurence of '>>'
    """
    # STUDENTS: Implement this function according to specification
    pass

    # BEGIN_REMOVE
    beg_pos = msg.find('<<')
    beg = msg[beg_pos+2:] # Need to take into account length of first marker!
    end_pos = beg.rfind('>>')
    whole = beg[:end_pos]
    return whole.strip()
    # END_REMOVE

def max_cols(table):
    """
    Returns: a list that contains the max value of each column

    For example:
    If table = [[1, 2, 3],
                [4, 5, 6],
                [7, 3, 9]]
    Then max_cols(A) returns [7, 5, 9]

    Precondition: table is a NONEMPTY 2D list of floats
    """

    # STUDENTS: Implement this function according to specification
    pass
    # Question taken from here: https://www.cs.cornell.edu/courses/cs1110/2017fa/exams/prelim2/prelim2-review.pdf

    #BEGIN_REMOVE
    # solution by I. Huang
    lst = []
    for col in range(len(table[0])):
        max_element = table[0][col]
        for row in range(len(table)):
            if max_element < table[row][col]:
                max_element = table[row][col]
        lst.append(max_element)
    return lst

    # Alt solution (from slides cited above):
    # result = table[0][:] # Need to make a copy
    # for row in table:
    #     for pos in range(len(row)):
    #         if row[pos] > result[pos]:
    #             result[pos] = row[pos]
    # return result
    #END_REMOVE

def wellformatted(input, labels):
    """
    Returns: True if 'input' is well-formatted with respect to 'labels',
        False otherwise.

    A string input is well-formatted if
        1. input is a list
        2. input has length at least two
        3. input’s first item is in the list labels
        4. each of the remaining items in input is either a string or a well-formatted list

    Precondition: labels is a (possibly empty) list.
    """

    # STUDENTS: Implement this function according to specification
    pass
    # Question taken from: Spring 2014 Prelim 2 (Questtion 4)

    # BEGIN_REMOVE
    if type(input) != list or len(input) < 2 or (input[0] not in labels):
        return False
    else:
        for item in input[1:]:
            if type(item) != str and not wellformatted(item,labels):
                return False
        return True

    # Alt Solution with different formatting in base case(s)
    # if type(input) != list:
    #     return False
    # elif len(input) < 2:
    #     return False
    # elif input[0] not in labels:
    #     return False
    # else:
    #     for item in input[1:]:
    #         if type(item) != str and not wellformatted(item,labels):
    #             return False
    #     return True
    #END_REMOVE


def duplicate_again(nums,a):
    """
    MODIFIES `nums` so that all occurrences of characters that are not `a` are
    duplicated (where each duplicate is adjacent to the original.)
    Does not return anything.

    Examples:
    If nums1 = [1,2,3,1], then after duplicate(nums1,1),
         the list that nums1 stores is *altered* to [1, 2, 2, 3, 3, 1].

    If nums2 = [1,2,3,1], then after duplicate(nums2, 4), nums2 is altered to
        [1, 1, 2 ,2, 3, 3, 1, 1]

    Preconditions:
      nums: list of ints
      a: an int
    """

    # STUDENTS: Implement this function according to specification
    pass
    # Question taken from optional while loop practice, found here:
        # https://cs1110.cs.cornell.edu/files/cs1110sp22/lab19-not-real-lab/while_funcs.py

    #BEGIN_REMOVE
    next_pos = 0
    while next_pos < len(nums):
        if nums[next_pos] == a:
            next_pos += 1
        else:
            next_pos += 1
            nums.insert(next_pos,nums[next_pos-1])
            next_pos += 1
    #END_REMOVE
